%
%
%  Time Projection Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Time Projection" icon="Matlab" tooltip="Project all channels of a time dataset to a single time point.">
%          <Command>MatlabXT::XTTimeProjection(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%  
%
%  Description:
%   
%   Project all channels of a time dataset to a single time point. 
%   For each voxel the intensity is computed using a MIP or mean projection
%       of all time points.
%   The projection mode is asked to the user. 
%
%

function XTTimeProjection(aImarisApplicationID, aProjectionMode)


if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end
vImarisApplication.DataSetPushUndo('Time Projection');

% get the data set 
vDataSet = vImarisApplication.GetDataSet.Clone;

vNumberOfTimePoints = vDataSet.GetSizeT;
vNumberOfChannels = vDataSet.GetSizeC;
vNumberOfSlices = vDataSet.GetSizeZ;

% MIP or Mean projection
if nargin<2
    vAnswer = questdlg('Please select the projection mode:', 'Time Projection', ...
        'MIP', 'Mean','Cancel', 'MIP');
    if strcmp(vAnswer, 'Cancel')
        return;
    else
        vProjectionMode = vAnswer;
    end
else
    vProjectionMode = aProjectionMode;
end

vTotalCount = vNumberOfChannels*vNumberOfSlices;
vProgressDisplay = waitbar(0, 'Projecting...');
vProgressCount = 0;

vImage = zeros(vDataSet.GetSizeX, vDataSet.GetSizeY, vNumberOfTimePoints);

for vChannel = 1:vNumberOfChannels
    for vSlice = 1:vNumberOfSlices
        
        for vTime = 1:vNumberOfTimePoints
            % typecast is done automatically
            vImage(:,:,vTime) = vDataSet.GetDataSliceFloats(vSlice-1, vChannel-1, vTime-1);
        end
        
        if strcmp(vProjectionMode, 'MIP')
            % MIP 
            vImageSlice = max(vImage, [], 3); 
        else
            % Mean 
            vImageSlice = mean(vImage, 3);  
        end
        
        if strcmp(vDataSet.GetType,'eTypeUInt8')
            vDataSet.SetDataSliceBytes(uint8(vImageSlice), vSlice-1, vChannel-1, 0);
        elseif strcmp(vDataSet.GetType,'eTypeUInt16')
            vDataSet.SetDataSliceShorts(uint16(vImageSlice), vSlice-1, vChannel-1, 0);
        else
            vDataSet.SetDataSliceFloats(single(vImageSlice), vSlice-1, vChannel-1, 0);
        end
        
        vProgressCount = vProgressCount + 1;

        waitbar(vProgressCount/vTotalCount)
    end
end

vImarisApplication.SetVisibleIndexT(0);
vDataSet.SetSizeT(1);

close(vProgressDisplay);

vImarisApplication.SetDataSet(vDataSet);
